<?php session_start(); 

// A set of commonly used functions
// Albert Teoh
// 27.3.2006

include_once 'constants.php';
include_once 'MysqlDB.php';

///////////////////////////////////////////////////////

// Initialise the input form variables
function init() {
  global $actions, $selected_obs;
	
  unset($actions);
  unset($selected_obs);

  unset($_SESSION['selected_obs']);
  unset($_SESSION['actions']);
}

///////////////////////////////////////////////////////

// Gets the parameter from the HTTP_GET_VARS or HTTP_POST_VARS assoc 
// array first. If not set, then gets it off the session
// variable
function get_param($param_name, $http_vars) {
	
		
  $param = "";
	
  if (isset($http_vars[$param_name])) {
    $param = $http_vars[$param_name];
		
    // Set the session variable
    $_SESSION[$param_name] = $param;
		
  }
  else {
    $param = $_SESSION[$param_name];
  }
	
  return $param;
}

// Create a tar of both observations and cals
function tarObsCalFiles($cart_obs, $all_results, $cals_within_mins, $db) {
  $all_files = "";
  $obs_cals  = array();
	
  $tar_filename = date("YmdHis");
	

  foreach ($cart_obs as $item) {
    $file = $item['filename'];
    $dir = $item['data_loc'];
    $path = $item['pathname'];

    // Query for all the respective cals
    $all_cals = query_cals($path, $cals_within_mins,$db);		
		
    // Need to create a symlink first because may not have write
    // access to the directory
    if (!file_exists(WORKING_DIR."/$file")) {
      if (file_exists("$dir/$file")) {
	symlink ("$dir/$file", WORKING_DIR."/$file");
      }
    }
    $all_files .= " $file";

    foreach ($all_cals as $cal) {
      $cal_file = $cal['filename'];
      $cal_dir = $cal['data_loc'];

      if (!file_exists(WORKING_DIR."/$cal_file")) {
	if (file_exists("$cal_dir/$cal_file")) {
	  symlink ("$cal_dir/$cal_file", WORKING_DIR."/$cal_file");
	}
      }
      $all_files .= " $cal_file";
    }

		
  }

  // -h to dereference the symlinks
  $command = "tar --dereference -cf $tar_filename.tar $all_files";

  // Change into the working directory first then tar
  chdir(WORKING_DIR);

  exec("$command", $output_array);
	
  if (file_exists(WORKING_DIR."/$tar_filename.tar")) {
    echo "Your requested archive set was successfully packaged. If you haven't been redirected to the tar
		file, you may access it directly by clicking on the following link:<br>
		<a href=\"".DATA_ARCHIVE_URL."/downloads/$tar_filename.tar\">".DATA_ARCHIVE_URL."/downloads/$tar_filename.tar</a><br>\n";
		
    return "$tar_filename.tar";
  }
}

// Create a tar of just the cals (no observations)
function tarCalFiles($cart_obs, $all_results, $cals_within_mins, $db) {
  $all_files = "";
  $obs_cals  = array();
	
  $tar_filename = date("YmdHis");
  
  foreach ($cart_obs as $index) {
	  
    // Query for all the respective cals
    $all_cals = query_cals( $all_results[$index]['pathname'], 
			    $cals_within_mins,$db);
		
    foreach ($all_cals as $cal) {
      $cal_file = $cal['filename'];
      $cal_dir = $cal['data_loc'];
			
      if (!file_exists(WORKING_DIR."/$cal_file")) {
	if (file_exists("$cal_dir/$cal_file")) {
	  symlink ("$cal_dir/$cal_file", WORKING_DIR."/$cal_file");
	}
      }
      $all_files .= " $cal_file";
    }	
  }
  
  // If we didn't find any cal files for any of the observations, return empty
  if( count( $all_files ) == 0 )
    return '';
	
  // -h to dereference the symlinks
  $command = "tar --dereference -cf $tar_filename.tar $all_files";

  // Change into the working directory first then tar
  chdir(WORKING_DIR);

  exec("$command", $output_array);

  if (file_exists(WORKING_DIR."/$tar_filename.tar")) {
    echo "Your requested archive set was successfully packaged. If you haven't been redirected to the tar
		file, you may access it directly by clicking on the following link:<br>
		<a href=\"".DATA_ARCHIVE_URL."/downloads/$tar_filename.tar\">".DATA_ARCHIVE_URL."/downloads/$tar_filename.tar</a><br>\n";
		
    return "$tar_filename.tar";
  }
  
  return '';
}

// Create a tar of all observation files (only)
function tarObsFiles($cart_obs, $all_results) {
  $all_files = "";
	
  $tar_filename = date("YmdHis");

  foreach ($cart_obs as $index) {
    $file = $all_results[$index]['filename'];
    $dir = $all_results[$index]['data_loc'];
		
    // Need to create a symlink first because may not have write
    // access to the directory
    if (!file_exists(WORKING_DIR."$file")) {
      if (file_exists("$dir/$file")) {
	symlink ("$dir/$file", WORKING_DIR."/$file");
      }
    }
    $all_files .= " $file";
		
  }

  // -h to dereference the symlinks
  $command = "tar --dereference -cf $tar_filename.tar $all_files";

  // Change into the working directory first then tar
  chdir(WORKING_DIR);

  exec("$command", $output_array);
	
  if (file_exists(WORKING_DIR."/$tar_filename.tar")) {
    echo "Your requested archive set was successfully packaged. If you haven't been redirected to the tar
		file, you may access it directly by clicking on the following link:<br>
		<a href=\"".DATA_ARCHIVE_URL."/downloads/$tar_filename.tar\">".DATA_ARCHIVE_URL."/downloads/$tar_filename.tar</a><br>\n";
		
    return "$tar_filename.tar";
  }
}


function hms2degs($hms_str) {

  list($hour, $min, $sec) = split(":", $hms_str);
	
  return ($hour + $min/60 + $sec/60)*HOURS2DEGS;
}

function dms2degs($dms_str) {
	
  list($deg, $min, $sec) = split(":", $dms_str) ;
	
  return $deg + $min/60 + $sec/60;
}


// Query the cals table for respective cals
//function query_cals($pulsar_name, $mjd, $data_type, $span_mins) {
function query_cals($pathname, $span_mins, $db) {

  // Query the obs table to get the data file we are finding cals for.
  // Only need to look in the obs table as there are no cals for the s70 data.
  //  Use the full pathname of the file to make sure we get the correct entry.

  $obsquery = "SELECT src_name,MJD,data_type FROM ".OBSERVATIONS_TABLE." WHERE concat(data_loc,'/',filename)='$pathname'";

  $db->DoQuery( $obsquery );

  $all_cals = Array();

  if( $db->FetchNumRows() > 0 )
  {
    $row = $db->FetchRowAsArray();

    $src = $row['src_name'];
    $mjd = $row['MJD'];
    $type= $row['data_type'];

    $mjd_max = mjd_add($mjd, $span_mins);
    $mjd_min = mjd_add($mjd, $span_mins*-1);

    $query = "SELECT * FROM ".CALS_TABLE." WHERE (MJD > $mjd_min) AND (MJD < $mjd_max) AND src_name LIKE '$src%' AND data_type = '$type' order by MJD";

    $db->DoQuery( $query );
    $all_cals = $db->GetLastResult();

    $_SESSION['cals'] = "cals";
  }

  return $all_cals;
}


function array2urlstring($arrayName, $array ) {
  $str = "";
  $brackets = urlencode("[]");
	
  if(is_array($array)){
    foreach ($array as $item) {
      $str .= "$arrayName$brackets=$item&";
    }
  }
  $str = trim($str, "&");
  return $str;
}



/**
 * array_diff2  Subtract rhs from lhs, both are arrays of arrays. This is used when you have an array of 
 *              records, each represented by an array, and you have a copy of one of those records that
 *              you wish to remove.
 *
 * @param lhs   The array to subtract from
 * @param rhs   The array to subtract
 * @example e   array_diff2( Array( Array( 'help' ), Array('me') ), Array( Array( 'help' ) ) )
 *              will return Array( Array( 'me' ) );
 **/
 
function array_diff2( $lhs, $rhs )
{
  $result = Array();
  foreach( $lhs as $next )
    if( !in_array( $next, $rhs ) )
      $result[] = $next;
  return $result;
}

?>
